# Generated by roxygen2: do not edit by hand

export(ctags_install)
export(ctags_test)
export(desc_stats)
export(dl_pkgstats_data)
export(extract_tarball)
export(loc_stats)
export(pkgstats)
export(pkgstats_cran_current_from_full)
export(pkgstats_fn_names)
export(pkgstats_fns_from_archive)
export(pkgstats_fns_update)
export(pkgstats_from_archive)
export(pkgstats_summary)
export(pkgstats_update)
export(plot_network)
export(rd_stats)
export(tags_data)
importFrom(memoise,memoise)
useDynLib(pkgstats, .registration = TRUE)
