% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cran-data-update.R
\name{pkgstats_update}
\alias{pkgstats_update}
\title{Update pkgstats` data on GitHub release}
\usage{
pkgstats_update(upload = TRUE)
}
\arguments{
\item{upload}{If \code{TRUE}, upload updated results to GitHub release.}
}
\value{
Local path to directory containing updated results.
}
\description{
This function is intended for internal rOpenSci use only. Usage by any
unauthorized users will error and have no effect unless run with \code{upload = FALSE}, in which case updated data will be created in the sub-directory
"pkgstats-results" of R's current temporary directory.
}
\seealso{
Other archive:
\code{\link[=dl_pkgstats_data]{dl_pkgstats_data()}},
\code{\link[=pkgstats_cran_current_from_full]{pkgstats_cran_current_from_full()}},
\code{\link[=pkgstats_fns_from_archive]{pkgstats_fns_from_archive()}},
\code{\link[=pkgstats_fns_update]{pkgstats_fns_update()}},
\code{\link[=pkgstats_from_archive]{pkgstats_from_archive()}}
}
\concept{archive}
